@@ -35,6 +35,7 @@
 #include "totem-private.h"
 
 static GtkFileFilter *filter_all = NULL;
+static GtkFileFilter *filter_subs = NULL;
 static GtkFileFilter *filter_supported = NULL;
 static GtkFileFilter *filter_audio = NULL;
 static GtkFileFilter *filter_video = NULL;
@@ -88,33 +89,88 @@ totem_pictures_dir (void)
 	return g_strdup (dir);
 }
 
-gboolean
-totem_is_media (const char *uri)
+static GnomeVFSVolume *
+totem_get_volume_for_uri (GnomeVFSVolumeMonitor *monitor, const char *path)
 {
-	if (uri == NULL)
-		return FALSE;
+	GnomeVFSVolume *vol;
+	GnomeVFSDeviceType type;
 
-	if (g_str_has_prefix (uri, "cdda:") != FALSE)
-		return TRUE;
-	if (g_str_has_prefix (uri, "dvd:") != FALSE)
-		return TRUE;
-	if (g_str_has_prefix (uri, "vcd:") != FALSE)
-		return TRUE;
-	if (g_str_has_prefix (uri, "cd:") != FALSE)
-		return TRUE;
+	vol = gnome_vfs_volume_monitor_get_volume_for_path (monitor, path);
+	if (vol == NULL)
+		return NULL;
 
-	return FALSE;
+	type = gnome_vfs_volume_get_device_type (vol);
+	if (type != GNOME_VFS_DEVICE_TYPE_AUDIO_CD
+	    && type != GNOME_VFS_DEVICE_TYPE_VIDEO_DVD
+	    && type != GNOME_VFS_DEVICE_TYPE_CDROM) {
+	    	gnome_vfs_volume_unref (vol);
+	    	vol = NULL;
+	}
+
+	return vol;
 }
 
-gboolean
+static char *
+totem_get_mountpoint_for_dvd (const char *uri)
+{
+	if (g_str_has_prefix (uri, "dvd://") == FALSE)
+		return NULL;
+	return g_strdup (uri + strlen ("dvd://"));
+}
+
+static char *
+totem_get_mountpoint_for_vcd (const char *uri)
+{
+	return NULL;
+}
+
+GnomeVFSVolume *
+totem_get_volume_for_media (const char *uri)
+{
+	GnomeVFSVolumeMonitor *monitor;
+	GnomeVFSVolume *ret;
+	char *mount;
+
+	if (uri == NULL)
+		return NULL;
+
+	mount = NULL;
+	ret = NULL;
+
+	if (g_str_has_prefix (uri, "dvd://") != FALSE)
+		mount = totem_get_mountpoint_for_dvd (uri);
+	else if (g_str_has_prefix (uri, "vcd:") != FALSE)
+		mount = totem_get_mountpoint_for_vcd (uri);
+	else if (g_str_has_prefix (uri, "file:") != FALSE)
+		mount = g_filename_from_uri (uri, NULL, NULL);
+
+	if (mount == NULL)
+		return NULL;
+
+	monitor = gnome_vfs_get_volume_monitor ();
+	ret = totem_get_volume_for_uri (monitor, mount);
+	g_free (mount);
+
+	return ret;
+}
+
+static gboolean
 totem_is_special_mrl (const char *uri)
 {
-	if (uri == NULL)
+	GnomeVFSVolume *vol;
+	gboolean retval;
+
+	if (uri == NULL || g_str_has_prefix (uri, "file:") != FALSE)
 		return FALSE;
 	if (g_str_has_prefix (uri, "dvb:") != FALSE)
 		return TRUE;
 
-	return totem_is_media (uri);
+	vol = totem_get_volume_for_media (uri);
+	retval = (vol != NULL);
+	if (vol != NULL)
+		gnome_vfs_volume_unref (vol);
+
+	return retval;
 }
 
 gboolean
@@ -178,7 +234,8 @@ totem_create_full_path (const char *path)
 
 static void
 totem_action_on_unmount (GnomeVFSVolumeMonitor *vfsvolumemonitor,
-		GnomeVFSVolume *volume, Totem *totem)
+			 GnomeVFSVolume *volume,
+			 Totem *totem)
 {
 	totem_playlist_clear_with_gnome_vfs_volume (totem->playlist, volume);
 }
@@ -189,9 +246,9 @@ totem_setup_file_monitoring (Totem *totem)
 	totem->monitor = gnome_vfs_get_volume_monitor ();
 
 	g_signal_connect (G_OBJECT (totem->monitor),
-			"volume_pre_unmount",
-			G_CALLBACK (totem_action_on_unmount),
-			totem);
+			  "volume_pre_unmount",
+			  G_CALLBACK (totem_action_on_unmount),
+			  totem);
 }
 
 /* List from xine-lib's demux_sputext.c */
@@ -201,34 +258,20 @@ static const char subtitle_ext[][4] = {
 	"sub",
 	"srt",
 	"smi",
-	"ssa"
+	"ssa",
+	"ass"
 };
 
-char *
-totem_uri_get_subtitle_uri (const char *uri)
+static char *
+totem_uri_get_subtitle_for_uri (const char *uri)
 {
+	GnomeVFSURI *vfsuri;
 	char *subtitle;
 	guint len, i, suffix;
-	GnomeVFSURI *vfsuri;
-
-	if (g_str_has_prefix (uri, "http") != FALSE) {
-		return NULL;
-	}
-
-	/* Does gnome-vfs support that scheme? */
-	vfsuri = gnome_vfs_uri_new (uri);
-	if (vfsuri == NULL)
-		return NULL;
-	gnome_vfs_uri_unref (vfsuri);
-
-	/* Has the user specified a subtitle file manually? */
-	if (strstr (uri, "#subtitle:") != NULL) {
-		return NULL;
-	}
 
         /* Find the filename suffix delimiter */
 	len = strlen (uri);
-	for (suffix = len-1; suffix > 0; suffix--) {
+	for (suffix = len - 1; suffix > 0; suffix--) {
 		if (uri[suffix] == G_DIR_SEPARATOR ||
                     (uri[suffix] == '/')) {
 			/* This filename has no extension, we'll need to 
@@ -269,6 +312,62 @@ totem_uri_get_subtitle_uri (const char *uri)
 	return NULL;
 }
 
+static char *
+totem_uri_get_subtitle_in_subdir (GnomeVFSURI *vfsuri, const char *subdir)
+{
+	char *filename, *subtitle, *fullpath_str;
+	GnomeVFSURI *parent, *fullpath, *directory;
+
+	parent = gnome_vfs_uri_get_parent (vfsuri);
+	directory = gnome_vfs_uri_append_path (parent, subdir);
+	gnome_vfs_uri_unref (parent);
+
+	filename = g_path_get_basename (gnome_vfs_uri_get_path (vfsuri));
+	fullpath = gnome_vfs_uri_append_string (directory, filename);
+	gnome_vfs_uri_unref (directory);
+	g_free (filename);
+
+	fullpath_str = gnome_vfs_uri_to_string (fullpath, 0);
+	gnome_vfs_uri_unref (fullpath);
+	subtitle = totem_uri_get_subtitle_for_uri (fullpath_str);
+	g_free (fullpath_str);
+
+	return subtitle;
+}
+
+char *
+totem_uri_get_subtitle_uri (const char *uri)
+{
+	GnomeVFSURI *vfsuri;
+	char *subtitle;
+
+	if (g_str_has_prefix (uri, "http") != FALSE) {
+		return NULL;
+	}
+
+	/* Has the user specified a subtitle file manually? */
+	if (strstr (uri, "#subtitle:") != NULL) {
+		return NULL;
+	}
+
+	/* Does gnome-vfs support that scheme? */
+	vfsuri = gnome_vfs_uri_new (uri);
+	if (vfsuri == NULL)
+		return NULL;
+
+	/* Try in the current directory */
+	subtitle = totem_uri_get_subtitle_for_uri (uri);
+	if (subtitle != NULL) {
+		gnome_vfs_uri_unref (vfsuri);
+		return subtitle;
+	}
+
+	subtitle = totem_uri_get_subtitle_in_subdir (vfsuri, "subtitles");
+	gnome_vfs_uri_unref (vfsuri);
+
+	return subtitle;
+}
+
 char*
 totem_uri_escape_for_display (const char *uri)
 {
@@ -335,6 +434,18 @@ totem_setup_file_filters (void)
 	gtk_file_filter_add_mime_type (filter_video, "application/x-cd-image");
 	gtk_file_filter_add_mime_type (filter_video, "application/x-cue");
 	g_object_ref (filter_video);
+
+	/* Subtitles files */
+	filter_subs = gtk_file_filter_new ();
+	gtk_file_filter_set_name (filter_subs, _("Subtitle files"));
+	gtk_file_filter_add_mime_type (filter_subs, "application/x-subrip"); /* *.srt */
+	gtk_file_filter_add_mime_type (filter_subs, "text/plain"); /* *.asc, *.txt */
+	gtk_file_filter_add_mime_type (filter_subs, "application/x-sami"); /* *.smi, *.sami */
+	gtk_file_filter_add_mime_type (filter_subs, "text/x-microdvd"); /* *.sub */
+	gtk_file_filter_add_mime_type (filter_subs, "text/x-mpsub"); /* *.sub */
+	gtk_file_filter_add_mime_type (filter_subs, "text/x-ssa"); /* *.ssa, *.ass */
+	g_object_ref (filter_subs);
+
 }
 
 void
@@ -390,6 +501,57 @@ totem_add_pictures_dir (GtkWidget *chooser)
 	gtk_file_chooser_add_shortcut_folder (GTK_FILE_CHOOSER (chooser), dir, NULL);
 }
 
+char *
+totem_add_subtitle (GtkWindow *parent, const char *path)
+{
+	GtkWidget *fs;
+	GConfClient *conf;
+	char *new_path;
+	char *subtitle = NULL;
+	gboolean set_folder;
+
+	fs = gtk_file_chooser_dialog_new (_("Select text subtitle"), 
+					  parent,
+					  GTK_FILE_CHOOSER_ACTION_OPEN,
+					  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					  GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+					  NULL);
+	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (fs), TRUE);
+	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (fs), filter_all);
+	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (fs), filter_subs);
+	gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (fs), filter_subs);
+
+	conf = gconf_client_get_default ();
+	set_folder = TRUE;
+	if (path != NULL) {
+		set_folder = gtk_file_chooser_set_current_folder_uri
+			(GTK_FILE_CHOOSER (fs), path);
+	} else {
+		new_path = gconf_client_get_string (conf, "/apps/totem/open_path", NULL);
+		if (new_path != NULL && *new_path != '\0') {
+			set_folder = gtk_file_chooser_set_current_folder_uri
+				(GTK_FILE_CHOOSER (fs), new_path);
+		}
+		g_free (new_path);
+	}
+	
+	if (set_folder == FALSE) {
+		gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (fs),
+						     g_get_home_dir ());
+	}
+	totem_add_default_dirs (GTK_FILE_CHOOSER (fs));
+
+	if (gtk_dialog_run (GTK_DIALOG (fs)) == GTK_RESPONSE_ACCEPT) {
+		char *filename;
+		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (fs));
+		subtitle = g_filename_to_uri (filename, NULL, NULL);
+		g_free(filename);		
+	}
+
+	gtk_widget_destroy (fs);
+	return subtitle;
+}
+
 GSList *
 totem_add_files (GtkWindow *parent, const char *path)
 {
@@ -398,6 +560,7 @@ totem_add_files (GtkWindow *parent, const char *path)
 	GSList *filenames;
 	char *mrl, *new_path;
 	GConfClient *conf;
+	gboolean set_folder;
 
 	fs = gtk_file_chooser_dialog_new (_("Select Movies or Playlists"),
 			parent,
@@ -415,18 +578,24 @@ totem_add_files (GtkWindow *parent, const char *path)
 	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (fs), FALSE);
 
 	conf = gconf_client_get_default ();
+	set_folder = TRUE;
 	if (path != NULL) {
-		gtk_file_chooser_set_current_folder_uri
+		set_folder = gtk_file_chooser_set_current_folder_uri
 			(GTK_FILE_CHOOSER (fs), path);
 	} else {
 		new_path = gconf_client_get_string (conf, "/apps/totem/open_path", NULL);
-		if (new_path != NULL && new_path != '\0') {
-			gtk_file_chooser_set_current_folder_uri
+		if (new_path != NULL && *new_path != '\0') {
+			set_folder = gtk_file_chooser_set_current_folder_uri
 				(GTK_FILE_CHOOSER (fs), new_path);
 		}
 		g_free (new_path);
 	}
 
+	/* We didn't manage to change the directory */
+	if (set_folder == FALSE) {
+		gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (fs),
+						     g_get_home_dir ());
+	}
 	totem_add_default_dirs (GTK_FILE_CHOOSER (fs));
 
 	response = gtk_dialog_run (GTK_DIALOG (fs));
